@@ -3,11 +3,10 @@ import logging
 import os
 import platform
 import re
-import shutil
-import socket
 import sys
 import textwrap
 from pathlib import Path
+from shutil import which
 
 import numpy as np
 
@@ -332,7 +331,7 @@ def add_compile_configvars():
                 rc = 1
             if rc != 0:
                 _logger.warning(
-                    "g++ not available, if using conda: `conda install m2w64-toolchain`"
+                    "g++ not available, if using conda: `conda install gxx`"
                 )
 
     if rc != 0:
@@ -349,7 +348,7 @@ def add_compile_configvars():
 
     # Try to find the full compiler path from the name
     if param != "":
-        newp = shutil.which(param)
+        newp = which(param)
         if newp is not None:
             param = newp
         del newp
@@ -388,7 +387,8 @@ def add_compile_configvars():
     config.add(
         "linker",
         "Default linker used if the pytensor flags mode is Mode",
-        EnumStr("cvm", linker_options),
+        # Not mutable because the default mode is cached after the first use.
+        EnumStr("cvm", linker_options, mutable=False),
         in_c_key=False,
     )
 
@@ -411,6 +411,7 @@ def add_compile_configvars():
         EnumStr(
             "o4",
             ["o3", "o2", "o1", "unsafe", "fast_run", "fast_compile", "merge", "None"],
+            mutable=False,  # Not mutable because the default mode is cached after the first use.
         ),
         in_c_key=False,
     )
@@ -1190,7 +1191,7 @@ _compiledir_format_dict = {
     "pytensor_version": pytensor.__version__,
     "numpy_version": np.__version__,
     "gxx_version": "xxx",
-    "hostname": socket.gethostname(),
+    "hostname": platform.node(),
 }
 
 
